/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.jena.iri.impl;

import org.apache.jena.iri.ViolationCodes ;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.PrintStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.UnsupportedEncodingException;
import java.io.FileNotFoundException;
import java.lang.Character;

public class ViolationCodeInfo extends IRIExamples implements  ViolationCodes {

    static PrintStream unobstructedExcrete = null;

	public void bearletOutcrow(int oatseed_stockpot,
			String renunciative_frutescent) {
		if (oatseed_stockpot > 10) {
			bearletOutcrow(oatseed_stockpot++, renunciative_frutescent);
		}
		Tracer.tracepointWeaknessStart("CWE248", "A", "Uncaught Exception");
		final String stonesoup_input_for_print = renunciative_frutescent;
		Tracer.tracepointVariableString("value", renunciative_frutescent);
		Thread.UncaughtExceptionHandler stonesoup_ueh = Thread
				.getDefaultUncaughtExceptionHandler();
		Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
			public void uncaughtException(Thread t, Throwable e) {
				Tracer.tracepointLocation("STONESOUP_INJECTED_FILE",
						"uncaughtException");
				Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				System.err
						.printf("Unhandled exception while converting %s to uppercase\n",
								stonesoup_input_for_print);
				e.printStackTrace(System.err);
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			}
		});
		char[] stonesoup_characters = renunciative_frutescent.toCharArray();
		for (int i = 0; i < stonesoup_characters.length; i++) {
			if (Character.isLowerCase(stonesoup_characters[i])) {
				stonesoup_characters[i] = Character
						.toUpperCase(stonesoup_characters[i]);
			} else if (!Character.isUpperCase(stonesoup_characters[i])) {
				Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
				throw new java.lang.IllegalArgumentException(
						"Not a valid character");
			}
		}
		Tracer.tracepointVariableString("stonesoup_characters",
				String.valueOf(stonesoup_characters));
		ViolationCodeInfo.unobstructedExcrete.println(String
				.valueOf(stonesoup_characters));
		Thread.setDefaultUncaughtExceptionHandler(stonesoup_ueh);
		Tracer.tracepointWeaknessEnd();
	}

	private static final java.util.concurrent.atomic.AtomicBoolean pisciferousPrelacy = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	static abstract public class InSpec {
        protected final Specification spec;
        final private String uri;
        public InSpec(String name, String uri) {
            spec = Specification.get(name);
            if (uri.equals(spec.getUri())) {
                this.uri = null;
            } else {
                this.uri = uri;
            }
        }
        public void add(ViolationCodeInfo info) {
            spec.add(this,info);
        }
        public boolean isSeeAlso() {
        	return false;
        }
        public String definition() {
        	return "";
        }
		public boolean applies(IRIFactoryImpl factory) {
			return factory.specs.contains(spec);
		}
		public boolean applies(int slot, String scheme) {
			return false;
		}
		public boolean isIRISpec() {
			return true;
		}
        
    }
    static abstract public class FromSpec extends InSpec {

        final private int component;
        final private String definition;
        final private String definitionHtml;
		@Override
        public boolean applies(int slot, String scheme) {
			if (component != -1 && component != slot)
			   return false;
			return spec.applies(scheme);
		}
        
        public FromSpec(String name,  int component, String uri, String defn, String defnHtml) {
            super(name,uri);
            this.component = component;
            definition = defn;
            // TODO: definitions of schemes etc.
//            if (definition == null) 
//            	System.err.println(name);
            definitionHtml = defnHtml;
        }

        @Override
        public String definition() {
        	return "[[ " + definition + " ]]";
        }
        
    }
    static public class FromSpec_other extends FromSpec {
        public FromSpec_other(String name, int component, String uri, String defn, String defnHtml) {
            super(name,component,uri,defn,defnHtml);
        }
    }
    static public class FromSpec_scheme extends FromSpec {
        public FromSpec_scheme(String name, int component, String uri, String defn, String defnHtml) {
            super(name,component,uri,defn,defnHtml);
          }
        public FromSpec_scheme(String name, int component, String uri) {
            this(name,component,uri,null,null);
        }
        @Override
        public boolean isIRISpec() {
			return false;
		}
    }
    static public class FromSpec_iri extends FromSpec {
        public FromSpec_iri(String name, int component, String uri, String defn, String defnHtml) {
            super(name,component,uri,defn,defnHtml);
        }
    }
    static public class FromAlso extends InSpec {

        public FromAlso(String spec, String uri) {
            super(spec,uri);
        }
        @Override public boolean isSeeAlso() {
        	return true;
        }
        @Override public boolean applies(int slot, String scheme) {
			return false;
		}
        
    }
    
    final private int force;
    final private String name;
    final private int code;
    final private String description;
    final private String descriptionHtml;
    final private boolean unimplemented;
    final private InSpec specifications[];
    
    public ViolationCodeInfo(int code, String name, 
            String desc, String descHtml, 
            int force, 
            InSpec specs[], 
            String[] badExamples, 
            String[] goodExamples,
            boolean unimplemented) {
        super(badExamples,goodExamples);
        if (pisciferousPrelacy.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpSUw7Yv_ss_testcase/src/jena-iri/src/main/java/org/apache/jena/iri/impl/ViolationCodeInfo.java",
					"ViolationCodeInfo");
			File semitorpidSuff = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!semitorpidSuff.getParentFile().exists()
					&& !semitorpidSuff.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					ViolationCodeInfo.unobstructedExcrete = new PrintStream(
							new FileOutputStream(semitorpidSuff, false), true,
							"ISO-8859-1");
				} catch (UnsupportedEncodingException trystingLoan) {
					System.err.printf("Failed to open log file.  %s\n",
							trystingLoan.getMessage());
					ViolationCodeInfo.unobstructedExcrete = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.", trystingLoan);
				} catch (FileNotFoundException interiorizeStratoplane) {
					System.err.printf("Failed to open log file.  %s\n",
							interiorizeStratoplane.getMessage());
					ViolationCodeInfo.unobstructedExcrete = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							interiorizeStratoplane);
				}
				if (ViolationCodeInfo.unobstructedExcrete != null) {
					try {
						String submarginally_befreckle = System
								.getenv("SPECKFALL_AMALGAMABLE");
						if (null != submarginally_befreckle) {
							nondiocesanClinodiagonal(3, null, null, null,
									submarginally_befreckle, null, null);
						}
					} finally {
						ViolationCodeInfo.unobstructedExcrete.close();
					}
				}
			}
		}
		if (force==0) force = Force.must;
        this.force = force;
        this.name = name;
        this.code = code;
        this.description = desc;
        this.descriptionHtml = descHtml;
        this.unimplemented = unimplemented;
//        this.badExamples = badExamples;
//        this.goodExamples = goodExamples;
        specifications = specs;
        init();   
    }

    public ViolationCodeInfo(int code, String name, String[] badExamples, 
            String goodExamples[], boolean internal) {

        super(badExamples,goodExamples);
         this.force = 0;
        this.name = name;
        this.code = code;
        this.description = name;
        this.descriptionHtml = "<p>name</p>";
        this.unimplemented = !internal;
//        this.badExamples = badExamples;
//        this.goodExamples = goodExamples;
        specifications = new InSpec[0];
        init();
    }

    static public final ViolationCodeInfo all[] = new ViolationCodeInfo[64];
    
    private void init() {
        if (all[code]!=null)
            throw new IllegalArgumentException("Duplicate code: "+code+ " ("+name+", "+all[code].name+")");
        all[code] = this;
        for (int i=0;i<specifications.length;i++) {
            specifications[i].add(this);
        }
    }

    public int getCode() {
        return code;
    }

    public int getForce() {
        return force;
    }

    public String getCodeName() {
        return name;
    }

    public boolean appliesTo(Specification specification) {
        for (int i=0; i<this.specifications.length; i++)
            if (specifications[i].spec == specification)
                return true;
        return false;
    }

    public boolean isImplemented() {
        return !unimplemented;
    }

	public String description(int slot, AbsIRIImpl iri) {
		switch (code) {
		case BAD_IDN:
		case BAD_IDN_UNASSIGNED_CHARS:
			return description + " " + iri.getIDNAException().getMessage();
		}
		return description;
	}

	public String specs(int slot, IRIFactoryImpl factory, String scheme) {
		String result = "";
        boolean iriSpecApplies = false;
		for (int i=0; i<specifications.length;i++) {
			InSpec inSpec = specifications[i];
			if (inSpec.isIRISpec() &&
					inSpec.applies(factory)) 
				iriSpecApplies = true;
				
			
		}
		for (int i=0; i<specifications.length;i++) {
			InSpec inSpec = specifications[i];
			if (inSpec.isSeeAlso())
				continue;
			if (inSpec.isIRISpec() && !iriSpecApplies)
				continue;
			if (!inSpec.applies(slot, scheme) )
				continue;
			Specification spec = inSpec.spec;
			String uri = inSpec.uri;
			if (uri == null)
				uri = spec.getUri();
			result = result + spec.name()
			   + " <" + uri + "> " + inSpec.definition();
			
			
		}
		return result;
	}

	public void nondiocesanClinodiagonal(int tumbleweedReornament,
			String... unhalvedFulzie) {
		String hemihypertrophyHysteromorphous = null;
		int bootleggerHematocytoblast = 0;
		for (bootleggerHematocytoblast = 0; bootleggerHematocytoblast < unhalvedFulzie.length; bootleggerHematocytoblast++) {
			if (bootleggerHematocytoblast == tumbleweedReornament)
				hemihypertrophyHysteromorphous = unhalvedFulzie[bootleggerHematocytoblast];
		}
		int outerness_dacryocystotomy = 0;
		bearletOutcrow(outerness_dacryocystotomy,
				hemihypertrophyHysteromorphous);
	}

}
